// Login.cpp - Login Dialog and Encryption

#include <PalmOS.h>
#include "Adven.h"
#include "Resource.h"

/*-------------------------------------------------------------------------*/

static void ListDrawFun (Int16 index, RectanglePtr bounds, Char** data)
{
	MRecord rec (index);
	WinDrawChars (rec.Data(), StrLen(rec.Data()), bounds->topLeft.x, bounds->topLeft.y);
}

/*-------------------------------------------------------------------------*/

static UInt16 GetSelectedItem()
{
	UInt16 idx = LstGetSelection ((ListPtr)GetObjectPtr (GameListSavedList));
	if (idx == noListSelection)
		FrmAlert (NoSelectionAlert);
	return idx;
}

/*-------------------------------------------------------------------------*/

static void GameListInitialize()
{
	FormPtr frm = FrmGetActiveForm();
	Boolean bSave = FrmGetFormId (frm) == SaveGameForm;
	UInt16 FrmId = bSave ? SaveGameSavedList : GameListSavedList;
	ListPtr list = (ListPtr)GetObjectPtr (FrmId);
	LstSetListChoices (list, NULL, DmNumRecords (AppDB));
	LstSetDrawFunction (list, ListDrawFun);

	// Let's see it
	FrmDrawForm (frm);
	if (bSave) {
		LstSetSelection (list, -1);
		FrmSetFocus (frm, FrmGetObjectIndex (frm, SaveGameNameField));
	}
}

/*-------------------------------------------------------------------------*/

static void GameListDelete()
{
	UInt16 idx = GetSelectedItem();
	if (idx == noListSelection) return;
	MRecord rec (idx);
	if (FrmCustomAlert (AskDeleteAlert, rec.Name(), NULL, NULL) == AskDeleteYes) {
		// rtd delete
	}
}

/*-------------------------------------------------------------------------*/

static void GameListRestore()
{
	UInt16 idx = GetSelectedItem();
	if (idx == noListSelection) return;
	FrmReturnToForm (0);
	StartNewGame (idx);
}

/*-------------------------------------------------------------------------*/

static void GameListSave()
{
	FieldPtr fld = GetObjectPtr (SaveGameNameField);
	Char* pName =  FldGetTextPtr (fld);
	if (pName == NULL || *pName == 0) {
		FrmAlert (NoNameAlert);
		return;
	}

	if (DBSaveGame ((char*)pName) != 0) {
		FrmCustomAlert (DBErrorAlert, pName, NULL, NULL);
		return;
	}
	FrmReturnToForm (0);
}

/*-------------------------------------------------------------------------*/
// This is the main even handler for LoginDlg

Boolean GameListHandleEvent (EventPtr event)
{
	Boolean handled = false;

	switch (event->eType) {
	case ctlSelectEvent:
		handled = true;
		switch (event->data.ctlSelect.controlID) {
		case GameListOKButton:
			GameListRestore();
			break;

		case SaveGameSaveButton:
			GameListSave();
			break;

		case GameListDeleteButton:
			GameListDelete();
			break;

		case SaveGameCancelButton:
		case GameListCancelButton:
			FrmReturnToForm (0);
			break;
		}
		break;
	
	case lstSelectEvent:
		if (event->data.lstSelect.listID == SaveGameSavedList) {
			MRecord rec (event->data.lstSelect.selection);
			FldReplaceText (GetObjectPtr (SaveGameNameField), rec.Name());
		}
		break;

	case frmOpenEvent:
		GameListInitialize();
		handled = true;
		break;
	}

	return handled;
}
