// Utility.cpp

#include <PalmOS.h>
#include "Utility.h"

// This file contains some useful utility functions

MemHandle ScreenData; // Output
static UInt32 ScreenLen;  // Num of chars in ScreenData

/*-------------------------------------------------------------------------*/

static Char* Resize (UInt32 nSize)
{
	if (ScreenData) {
		if (MemHandleResize (ScreenData, nSize) != 0)
			return NULL;
	} else {
		ScreenData = MemHandleNew (nSize);
		if (ScreenData == NULL) return NULL;
	}

	ScreenLen = nSize - sizeof(Char);
	return (Char *)MemHandleLock (ScreenData);
}

/*-------------------------------------------------------------------------*/

void ScreenAdd (const Char* pStr, UInt32 nChars)
{
	UInt32 lenOld = ScreenData ? ScreenLen : 0;
	Char* pData = Resize (lenOld + nChars + sizeof(Char));
	if (pData) {
		MemMove (pData+lenOld, pStr, nChars);
		pData[lenOld+nChars] = '\0';
		MemHandleUnlock (ScreenData);
	}
}

void ScreenAdd (const Char* pStr)
{
	ScreenAdd (pStr, StrLen (pStr));
}

/*=========================================================================*/

MStreamOut::MStreamOut (UInt32 nSize)
{
	m_nAlloc = nSize;
	m_nLen = 0;
	m_hData = NULL;
	m_nError = 0;
	if (m_nAlloc) {
		char* pData = Resize (m_nAlloc);
		if (pData) {
			MemHandleUnlock (m_hData);
			m_nLen = 0;
		} else
			m_nAlloc = 0;
	}
}

/*-------------------------------------------------------------------------*/

char* MStreamOut::Resize (UInt32 nSize)
{
	if (m_hData) {
		if (nSize > m_nAlloc &&
			MemHandleResize (m_hData, nSize) != 0)
			return NULL;
	} else {
		m_hData = MemHandleNew (nSize);
		if (m_hData == NULL) return NULL;
	}

	m_nLen = nSize;
	return (char*)MemHandleLock (m_hData);
}

/*-------------------------------------------------------------------------*/

void MStreamOut::Put (int val)
{
	if (m_nError) return; // stop writing after error

	// keep, because Resize changes size (obviously!)
	UInt32 lenOld = m_nLen;
	char* pData = Resize (lenOld + sizeof(val));
	if (pData) {
		MemMove (pData+lenOld, &val, sizeof(val));
		MemHandleUnlock (m_hData);
	} else
		m_nError = 1;
}

/*-------------------------------------------------------------------------*/

void MStreamOut::Put (Char* pStr)
{
	if (m_nError) return; // stop writing after error

	UInt32 lenNew = StrLen (pStr) + 1;
	UInt32 lenOld = m_nLen;
	char* pData = Resize (lenOld + lenNew);
	if (pData) {
		MemMove (pData+lenOld, pStr, lenNew);
		MemHandleUnlock (m_hData);
	} else
		m_nError = 1;
}

/*=========================================================================*/

MStreamIn::MStreamIn (const char* pData, UInt32 nLen)
{
	m_nLen = nLen;
	m_pData = pData;
	m_nError = 0;
}

/*-------------------------------------------------------------------------*/
// returns the pointer to start of the skipped memory
// sets the error flag if not enough mem.

const char* MStreamIn::EatData (UInt32 nLen)
{
	if (m_nError || m_nLen < nLen) {
		m_nError = 1;
		return NULL;
	}

	m_nLen += nLen;
	const char* pData = m_pData;
	m_pData += nLen;

	return pData;
}

int MStreamIn::GetW()
{
	int val = 0;
	const char* pData = EatData (sizeof (val));
	if (pData)
		MemMove (&val, pData, sizeof (val));
	return val;
}

