// Utility.h


extern MemHandle ScreenData; // Output

/*-------------------------------------------------------------------------*/

void ScreenAdd (const Char* pStr, UInt32 nChars);
void ScreenAdd (const Char* pStr);

class CString
{
public:
	CString (UInt32 nSize = 0);
	~CString() {Empty();}

	operator Char*() const {return m_pData;}
	const CString& operator+=(const Char* pStr);
	const CString& operator+=(Char ch);
	const CString& operator=(const Char* pStr);

//	Char operator[](UInt16 nIndex) const
//		{return m_pData[nIndex];}

	// Some std functions
	UInt32 GetLength() const {return StrLen (m_pData);}
	Boolean IsEmpty() const {return m_pData == NULL || *m_pData == 0;}
	void Empty();

protected:
	Char* m_pData;
	MemHandle m_hMem;

	Boolean Resize (UInt32 nSize);
};

/*-------------------------------------------------------------------------*/

class MResStr
{
public:
	MResStr (UInt16 rcID = 0)
	{
		m_pData = NULL;
		if (rcID) LoadString (rcID);
	}

	~MResStr() 
	{
		if (m_pData) MemPtrUnlock (m_pData);
		// Is it really necessary to call "DmReleaseResource"? 
		// Many functions in the Palm apps don't call it. The problem is we 
		// don't have a handle here, because it's not stored.
	}

	operator Char*() const {return (Char*)m_pData;}

	Char* LoadString (UInt16 rcID) 
	{
		if (m_pData) MemPtrUnlock (m_pData);
		m_pData = MemHandleLock (DmGetResource (strRsc, rcID));
		return (Char*)m_pData;
	}

protected:
	MemPtr m_pData;
};

/*-------------------------------------------------------------------------*/

class MStreamOut
{
public:
	MStreamOut (UInt32 nSize = 0);
	~MStreamOut() {if (m_hData) MemHandleFree (m_hData);}

	void Put (int val);
	void Put (Char* pStr);
	UInt32 GetLength() const {return m_nLen;}
	MemHandle GetHandle() const {return m_hData;}
	UInt16 Error() {return m_nError;}

protected:
	UInt32 m_nLen, m_nAlloc;
	UInt16 m_nError;
	MemHandle m_hData;

	char* Resize (UInt32 nSize);
};

/*-------------------------------------------------------------------------*/

class MStreamIn
{
public:
	MStreamIn (const char* pData, UInt32 nLen);

	int GetW();
	UInt32 GetLength() const {return m_nLen;}
	const char* EatData (UInt32 nLen);
	const char* GetRaw() const {return m_pData;}
	UInt16 Error() {return m_nError;}

protected:
	UInt32 m_nLen;
	UInt16 m_nError;
	const char* m_pData;
};
